﻿// ----------------------------------public ----------------------------------------------------------------------------------
// <copyright file="EventData.cs" company="GW2.Net Coding Team">
//   This product is licensed under the GNU General Public License version 2 (GPLv2) as defined on the following page: http://www.gnu.org/licenses/gpl-2.0.html
// </copyright>
// <summary>
//   Defines the EventData type.
// </summary>
// --------------------------------------------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Linq;

using GW2DotNET.V1.Infrastructure;
using GW2DotNET.V1.World.Models;

namespace GW2DotNET.V1.World.DataProvider
{
    using System.Collections;

    /// <summary>
    /// A class for retrieving information about the
    /// state of world events.
    /// </summary>
    /// <remarks>
    /// Note that unlike MapData and WorldData, this class does NOT
    /// implement IEnumerable. This is intentional, because we do not expose
    /// the event names list to the caller. We only return events objects
    /// that have status information, not just name-id mappings.
    /// </remarks>
    public class EventData : IEnumerable<GwEvent>
    {
        /// <summary>
        /// Keep a pointer to our WorldManager here for ID resolution.
        /// </summary>
        private readonly GW2ApiManager gw2ApiManager;

        /// <summary>
        /// The events names will be retrieved in this language
        /// </summary>
        private readonly Language language;

        /// <summary>
        /// Cache the event names here
        /// </summary>
        private Dictionary<Guid, string> eventNamesCache;

        /// <summary>
        /// Initializes a new instance of the <see cref="EventData"/> class.
        /// This should only be called by WorldManager.
        /// </summary>
        /// <param name="language">The language in which to return names</param>
        /// <param name="gw2ApiManager">An instance of GW2ApiManager to use for ID resolution</param>
        internal EventData(Language language, GW2ApiManager gw2ApiManager)
        {
            this.language = language;
            this.gw2ApiManager = gw2ApiManager;
        }
        
        /// <summary>
        /// Gets the EventNames dictionary.
        /// This is not exposed to callers. It is for internal
        /// usage only.
        /// </summary>
        internal Dictionary<Guid, string> EventNames
        {
            get
            {
                if (this.eventNamesCache == null)
                {
                    var arguments = new List<KeyValuePair<string, object>>
                                        {
                                            new KeyValuePair<string, object>(
                                                "lang", this.language)
                                        };

                    var namesResponse = ApiCall.GetContent<List<Dictionary<string, string>>>("event_names.json", arguments, ApiCall.Categories.World);

                    // Create a new Dictionary to hold the names,
                    // whereas the key is the event id and the valus the event name.
                    this.eventNamesCache = new Dictionary<Guid, string>();

                    // Iterate through the namesResponse,
                    // so we can throw away that damn List<Dictionary<string,string>>! *blargh*
                    foreach (var eventName in namesResponse)
                    {
                        var id = new Guid();
                        
                        var name = string.Empty;

                        foreach (var variable in eventName)
                        {
                            if (variable.Key == "id")
                            {
                                id = new Guid(variable.Value);
                            }
                            else
                            {
                                name = variable.Value;
                            }
                        }

                        this.eventNamesCache.Add(id, name);
                    }
                }

                return this.eventNamesCache;
            }
        }

        /// <summary>
        /// Gets the events.
        /// </summary>
        private IEnumerable<GwEvent> Events
        {
            get
            {
                return this.GetEvents();
            }
        }

        /// <summary>
        /// Gets a collection of <see cref="GwEvent"/>s pre-sorted with a <see cref="GwWorld"/>.
        /// </summary>
        /// <param name="world">The world to pre-sort the events.</param>
        /// <returns>The collection of events.</returns>
        public IEnumerable<GwEvent> this[GwWorld world]
        {
            get
            {
                return this.GetEvents(world.Id);
            }
        }

        /// <summary>
        /// Gets all events for a particular map on all worlds.
        /// </summary>
        /// <param name="map"></param>
        /// <returns></returns>
        public IEnumerable<GwEvent> this[GwMap map]
        {
            get
            {
                return this.GetEvents(null, map.Id);
            }
        }

        /// <summary>
        /// Gets one event for all worlds.
        /// </summary>
        /// <param name="eventId"></param>
        /// <returns></returns>
        public IEnumerable<GwEvent> this[Guid eventId]
        {
            get
            {
                return this.GetEvents(null, null, eventId);
            }
        }

        /// <summary>
        /// Returns an enumerator that iterates through the collection.
        /// </summary>
        /// <returns>
        /// A <see cref="T:System.Collections.Generic.IEnumerator`1"/> that can be used to iterate through the collection.
        /// </returns>
        public IEnumerator<GwEvent> GetEnumerator()
        {
            return this.Events.GetEnumerator();
        }

        /// <summary>
        /// Returns an enumerator that iterates through a collection.
        /// </summary>
        /// <returns>
        /// An <see cref="T:System.Collections.IEnumerator"/> object that can be used to iterate through the collection.
        /// </returns>
        IEnumerator IEnumerable.GetEnumerator()
        {
            return this.Events.GetEnumerator();
        }

        /// <summary>
        /// Gets all events for all maps on all worlds.
        /// </summary>
        /// <returns>An <see cref="T:System.Collections.IEnumerable"/></returns>
        private IEnumerable<GwEvent> GetEvents(int? WorldID = null, int? MapID = null, Guid? EventID = null)
        {
            List<KeyValuePair<string, object>> parameters = new List<KeyValuePair<string, object>>();
            if (WorldID != null)
                parameters.Add(new KeyValuePair<string, object>("world_id", WorldID));
            if (MapID != null)
                parameters.Add(new KeyValuePair<string, object>("map_id", MapID));
            if (EventID != null)
                parameters.Add(new KeyValuePair<string, object>("event_id", EventID));

            return ApiCall.GetContent<Dictionary<string, List<GwEvent>>>("events.json", parameters, ApiCall.Categories.World)["events"].Select(evnt => evnt.ResolveIDs(this.gw2ApiManager));
        }
    }
}
